<?php

/**
 * @file
 * Admin page
 */

/**
 * Create page with information about setting of Google reCAPTCHA
 * Is keys were input?
 * List of all protected forms
 * Statistics data - if was enabled (default)
 * Some module settings
 * Where to get additional information about Google reCAPTCHA
 *
 * @return string
 */
function g_captcha_overview() {
  $conf = variable_get('g_captcha');
  $output = theme_render_template(
    drupal_get_path('module', 'g_captcha') . '/g-captcha-admin.tpl.php', $conf
  );
  return $output;
}

/**
 * Create form for managing Google reCAPTCHA keys
 * and different settings
 *
 * @param $form
 * @param &$form_state
 *
 * @return array
 */
function g_captcha_settings_form($form, &$form_state) {
  $tune = variable_get('g_captcha');

  $form['api_keys'] = array(
    '#type' => 'fieldset',
    '#title' => t('API Keys'),
    '#tree' => FALSE,
    '#description' => t('These can be retrieved from the
      <a href="@google-recaptcha-site" target="_blank">Google reCAPTCHA site.
      </a>.', array(
        '@google-recaptcha-site' => 'https://www.google.com/recaptcha'
      )
    ),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE
  );
  $form['api_keys']['public_key'] = array(
    '#title' => t('Public Key'),
    '#type' => 'textfield',
    '#default_value' => empty($tune['public_key']) ? '' : $tune['public_key'],
    // may be without additional check?
    '#size' => 40,
    '#maxlength' => 40,
    '#required' => TRUE,
  );
  $form['api_keys']['secret_key'] = array(
    '#title' => t('Secret Key'),
    '#type' => 'textfield',
    '#default_value' => empty($tune['secret_key']) ? '' : $tune['secret_key'],
    '#size' => 40,
    '#maxlength' => 40,
    '#required' => TRUE,
  );
  $form['widget_display'] = array(
    '#type' => 'fieldset',
    '#title' => t('Widget Defaults'),
    '#tree' => FALSE,
    '#collapsible' => TRUE,
    '#collapsed' => FALSE
  );
  $form['widget_display']['widget_type'] = array(
    '#title' => t('Type'),
    '#type' => 'select',
    '#options' => array(
      'normal' => 'Invisible reCAPTCHA',
      'compact' => 'reCAPTCHA V2'
    ),
    '#default_value' => empty($tune['settings']['widget_size']) ? 'normal' : $tune['settings']['widget_size'],
  );
  $form['widget_display']['language'] = array(
    '#title' => t('Language'),
    '#description' => t('Enter the language code that can be found here: https://developers.google.com/recaptcha/docs/language'),
    '#type' => 'select',
    '#default_value' => empty($tune['settings']['language']) ? 'en' : $tune['settings']['language'],
    '#options' => g_captcha_language_code_options(),
  );
  $form['widget_display']['widget_size'] = array(
    '#title' => t('Size'),
    '#type' => 'select',
    '#options' => array(
      'normal' => 'Normal',
      'compact' => 'Compact'
    ),
    '#default_value' => empty($tune['settings']['widget_size']) ? 'normal' : $tune['settings']['widget_size'],
  );
  $form['widget_display']['widget_theme'] = array(
    '#title' => t('Theme'),
    '#type' => 'select',
    '#options' => array(
      'light' => 'Light',
      'dark' => 'Dark'
    ),
    '#default_value' => empty($tune['settings']['widget_theme']) ? 'dark' : $tune['settings']['widget_theme'],
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save')
  );

  return $form;
}

/**
 * Validate handler
 *
 * @param $form
 * @param $form_state
 */
function g_captcha_settings_form_validate($form, &$form_state) {
  if (strlen($form_state['values']['public_key']) != 40) {
    form_set_error('public_key', t('Length of public key must be 40 symbols long.'));
  }
  if (strlen($form_state['values']['secret_key']) != 40) {
    form_set_error('secret_key', t('Length of secret key must be 40 symbols long.'));
  }
}

/**
 * Submit handler
 *
 * @param $form
 * @param $form_state
 */
function g_captcha_settings_form_submit($form, &$form_state) {
  $conf = variable_get('g_captcha');
  $conf['public_key'] = $form_state['values']['public_key'];
  $conf['secret_key'] = $form_state['values']['secret_key'];
  $conf['settings']['widget_type'] = $form_state['values']['widget_type'];
  $conf['settings']['widget_size'] = $form_state['values']['widget_size'];
  $conf['settings']['widget_theme'] = $form_state['values']['widget_theme'];
  $conf['settings']['language'] = $form_state['values']['language'];

  variable_set('g_captcha', $conf);
  drupal_set_message(t('G Captcha settings were saved'));
}

/**
 * Create form for managing list of protected forms.
 *
 * @param $form
 * @param $form_state
 *
 * @return array
 */
function g_captcha_forms_form($form, &$form_state) {
  $conf = variable_get('g_captcha');
  // Configuration of which forms to protect, with what challenge.
  $form['captcha_form_protection'] = array(
    '#type' => 'fieldset',
    '#title' => t('Form protection'),
    '#description' => t("Select the challenge type you want for each of the listed forms (identified by their so called <em>form_id</em>'s)."),
  );
  // List known form_ids.
  $form['captcha_form_protection']['captcha_form_id_overview'] = array(
    '#theme' => 'g_captcha_forms_instances',
    '#tree' => TRUE,
  );
  $form['captcha_form_protection']['captcha_form_id_overview']['captcha_captcha_points'] = array();

  $types = _g_captcha_types();
  $sizes = _g_captcha_sizes(TRUE);
  $themes = _g_captcha_themes(TRUE);
  $forms = g_captcha_get_instances();
  $instances = (array) $forms;
  foreach($instances as $instance) {

    $elem = array();
    $elem['form_id'] = array(
      '#markup' => $instance->form_id,
    );

    if (isset($instance->captcha_type)) {
      $captcha_type = $instance->captcha_type;
    }
    else {
      $captcha_type = 'default';
    }

    $captcha_settings = json_decode(
      implode(
        (array) $instance->captcha_settings
      )
    );

    if (isset($captcha_settings->size)) {
      $captcha_size = $captcha_settings->size;
    }
    else {
      $captcha_size = 'normal';
    }

    if (isset($captcha_settings->theme)) {
      $captcha_theme = $captcha_settings->theme;
    }
    else {
      $captcha_theme = 'default';
    }

    $elem['captcha_type'] = array(
      '#type' => 'select',
      '#default_value' => $captcha_type,
      '#options' => $types
    );
    $elem['captcha_size'] = array(
      '#type' => 'select',
      '#default_value' => $captcha_size,
      '#options' => $sizes
    );
    $elem['captcha_theme'] = array(
      '#type' => 'select',
      '#default_value' => $captcha_theme,
      '#options' => $themes
    );
    $ops = array();
      if ($instance->export_type & EXPORT_IN_DATABASE) {
        $ops[] = l(t('revert'), "admin/config/spam_protection/g_captcha/forms/form/{$instance->form_id}/delete");
      }
    $elem['operations'] = array('#markup' => implode(", ", $ops));

    $form['captcha_form_protection']['captcha_form_id_overview']['captcha_captcha_points'][$instance->form_id] = $elem;

  }
  // Form items for new form_id.
  $form['captcha_form_protection']['captcha_form_id_overview']['captcha_new_captcha_point'] = array();
  // Textfield for form_id.
  $form['captcha_form_protection']['captcha_form_id_overview']['captcha_new_captcha_point']['form_id'] = array(
    '#type' => 'textfield',
    '#size' => 16,
  );
  // Select widget for CAPTCHA type.
  $form['captcha_form_protection']['captcha_form_id_overview']['captcha_new_captcha_point']['captcha_type'] = array(
    '#type' => 'select',
    '#default_value' => 'default',
    '#options' => $types,
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save')
  );

  $form['#tree'] = TRUE;

  return $form;
}

/**
 * Submit handler
 *
 * @param $form
 * @param $form_state
 */
function g_captcha_forms_form_submit($form, &$form_state) {
  $conf = variable_get('g_captcha');

  drupal_set_message('Settings were saved.');

  if(isset($form_state['values']['captcha_form_protection']['captcha_form_id_overview']['captcha_captcha_points'])) {
    // Load existing data.
    $instances = g_captcha_get_instances();
    $forms = $form_state['values']['captcha_form_protection']['captcha_form_id_overview']['captcha_captcha_points'];
    foreach ($forms as $form_id => $data) {
      // If this is an in-code captcha point and its settings are unchanged,
      // don't save to the database.
      if (module_exists('ctools') && isset($instances[$form_id])) {
        // Parse module and captcha_type from submitted values.
        if (is_string($data['captcha_type']) && substr_count($data['captcha_type'], '/') == 1) {
          list($module, $captcha_type) = explode('/', $data['captcha_type']);
        }
        else {
          $module = '';
          $captcha_type = $data['captcha_type'];
        }

        $point = $instances[$form_id];
        if ($point->export_type & EXPORT_IN_CODE && !($point->export_type & EXPORT_IN_DATABASE) && $point->module == $module && $point->captcha_type == $captcha_type) {
          continue;
        }
      }
      $data['captcha_settings'] = array();
      $data['captcha_settings']['size'] = $data['captcha_size'];
      $data['captcha_settings']['theme'] = $data['captcha_theme'];
      g_captcha_form_id_setting($form_id, $data['captcha_type'], $data['captcha_settings']);

    }
  }

  variable_set('g_captcha', $conf);

}

/**
 * Validation handler for captcha_admin_settings form.
 */
function g_captcha_forms_form_validate($form, $form_state) {
  $form_id = $form_state['values']['captcha_form_protection']['captcha_form_id_overview']['captcha_new_captcha_point']['form_id'];
  if (!preg_match('/^[a-z0-9_]*$/', $form_id)) {
    form_set_error('captcha_form_id_overview][captcha_new_captcha_point][form_id', t('Illegal form_id'));
  }
}

/**
 * Helper function for adding/updating a CAPTCHA point.
 *
 * @param string $form_id
 *   the form ID to configure.
 *
 * @param string $captcha_type
 *   the setting for the given form_id, can be:
 *   - 'none' to disable CAPTCHA,
 *   - 'default' to use the default challenge type
 *   - NULL to remove the entry for the CAPTCHA type
 *   - something of the form 'image_captcha/Image'
 *   - an object with attributes $captcha_type->module and $captcha_type->captcha_type
 */
function g_captcha_form_id_setting($form_id, $captcha_type, $captcha_settings) {
    db_merge('g_captcha_instances')
      ->key(array('form_id' => $form_id))
      ->fields(array('captcha_type' => $captcha_type))
      ->fields(array('captcha_settings' => json_encode($captcha_settings)))
      ->execute();
}

/**
 * Central handler for CAPTCHA point administration (adding, disabling, deleting).
 */
function g_captcha_instance_admin($form_id = NULL, $op = NULL) {

  // If $captcha_point_form_id and action $op given: do the action.
  if ($form_id) {
    switch ($op) {
      case 'delete':
        return drupal_get_form('g_captcha_instance_disable_confirm', $form_id, TRUE);
    }
  }
  // Return add form for CAPTCHA point.
  return drupal_get_form('g_captcha_forms_form');
}


/**
 * Confirm dialog for disabling/deleting a CAPTCHA point.
 */
function g_captcha_instance_disable_confirm($form, &$form_state, $form_id, $delete) {
  $form = array();
  $form['instance_id'] = array(
    '#type' => 'value',
    '#value' => $form_id,
  );
  $form['g_captcha_delete'] = array(
    '#type' => 'value',
    '#value' => $delete,
  );
  if ($delete) {
    $message = t('Are you sure you want to delete the CAPTCHA for form_id %form_id?', array('%form_id' => $form_id));
    $yes = t('Delete');
  }
  return confirm_form($form, $message, 'admin/config/spam_protection/g_captcha/forms', '', $yes);
}

/**
 * Submission handler of CAPTCHA point disabling/deleting confirm_form.
 */
function g_captcha_instance_disable_confirm_submit($form, &$form_state) {
  $form_id = $form_state['values']['instance_id'];
  $delete = $form_state['values']['g_captcha_delete'];
  if ($delete) {
    db_delete('g_captcha_instances')
    ->condition('form_id', $form_id)
    ->execute();
    drupal_set_message(t('Deleted CAPTCHA for form %form_id.', array('%form_id' => $form_id)));
  }
  $form_state['redirect'] = 'admin/config/spam_protection/g_captcha/forms';
}
